/**
 *
 * Project: Eight Track, File: Mp3UploadResults.java
 *
 * Copyright 2019-2019 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Eight-Track(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.discord.eighttrack.handlers;

import org.apache.tika.metadata.Metadata;

public class Mp3UploadResults {
  
  private enum MetaDataField {
    GENRE("xmpDM:genre"),
    COMPOSER("xmpDM:composer"),
    ALBUM("xmpDM:album"),
    TRACK_NUMBER("xmpDM:trackNumber"),
    DISC_NUMBER("xmpDM:discNumber"),
    ARTIST("xmpDM:artist"),
    TITLE("title"),
    RELEASE_DATE("xmpDM:releaseDate"),
    DURATION("xmpDM:duration");
    
    private String value;
    
    private MetaDataField(String value) {
      this.value = value;
    }
    
    public String getValue() {
      return this.value;
    }
  }
  
  private String genre;
  private String composure;
  private String album;
  private String trackNumber;
  private String discNumber;
  private String artist;
  private String title;
  private String releaseDate;
  private Long duration;
  
  private byte[] rawData;
  
  public Mp3UploadResults( Metadata metadata, byte[] data) {
    parseResults(metadata);
    this.rawData = data;
  }

  private void parseResults(Metadata metadata) {
    genre = metadata.get(MetaDataField.GENRE.getValue());
    composure = metadata.get(MetaDataField.COMPOSER.getValue());
    album = metadata.get(MetaDataField.ALBUM.getValue());
    trackNumber = metadata.get(MetaDataField.TRACK_NUMBER.getValue());
    discNumber = metadata.get(MetaDataField.DISC_NUMBER.getValue());
    artist = metadata.get(MetaDataField.ARTIST.getValue());
    title = metadata.get(MetaDataField.TITLE.getValue());
    releaseDate = metadata.get(MetaDataField.RELEASE_DATE.getValue());
    
    String durationStr = metadata.get(MetaDataField.DURATION.getValue()).trim();
    if (durationStr != null) {
      duration = Double.valueOf(metadata.get(MetaDataField.DURATION.getValue())).longValue();
    }
  }
  
  public final byte[] getData() {
    return this.rawData;
  }

  /**
   * @return the genre
   */
  public final String getGenre() {
    return genre;
  }

  /**
   * @return the composure
   */
  public final String getComposure() {
    return composure;
  }

  /**
   * @return the album
   */
  public final String getAlbum() {
    return album;
  }

  /**
   * @return the trackNumber
   */
  public final String getTrackNumber() {
    return trackNumber;
  }

  /**
   * @return the discNumber
   */
  public final String getDiscNumber() {
    return discNumber;
  }

  /**
   * @return the artist
   */
  public final String getArtist() {
    return artist;
  }

  /**
   * @return the title
   */
  public final String getTitle() {
    return title;
  }

  /**
   * @return the releaseDate
   */
  public final String getReleaseDate() {
    return releaseDate;
  }

  /**
   * @return the duration
   */
  public final Long getDuration() {
    return duration;
  }
  
  public final String getDurationFormat() {
    return dateFormat(duration);
  }
  
  public boolean validFile() {
    return getTitle() != null && !getTitle().isEmpty();
  }
  
  @Override
  public String toString() {
    StringBuilder sb = new StringBuilder();
    if (getTitle() != null)
      sb.append(String.format("%-10s: %s%n", "Title", getTitle()));
    
    if (getArtist() != null)
      sb.append(String.format("%-10s: %s%n", "Artist", getArtist()));
    
    if (getAlbum() != null)
      sb.append(String.format("%-10s: %s%n", "Album", getAlbum()));
    
    if (getTrackNumber() != null)
      sb.append(String.format("%-10s: %s%n", "Track", getTrackNumber()));
    
    if (getDiscNumber() != null)
      sb.append(String.format("%-10s: %s%n", "Disc", getDiscNumber()));
    
    if (getReleaseDate() != null)
      sb.append(String.format("%-10s: %s%n", "Year", getReleaseDate()));
    
    if (getGenre() != null)
      sb.append(String.format("%-10s: %s%n", "Genre", getGenre()));
    
    if (duration != null)
      sb.append(String.format("%-10s: %s%n", "Duration", dateFormat(getDuration())));
    
    return sb.toString();
  }
  
  public void clear() {
    rawData = new byte[0];
  }
  
  private String dateFormat(long timeInMilliSeconds) {
    long seconds = timeInMilliSeconds / 1000;
    long minutes = seconds / 60;
    long hours = minutes / 60;
    long days = hours / 24;
    
    long sec = seconds % 60;
    long min =  minutes % 60;
    long hr  = hours % 24;
    
    String time = "" + sec;
    if (min > 0) time = min + ":" + time;
    if (hr > 0) time = hr + ":" + time;
    if (days > 0) time = days + ":" + time;
    
    return time;
  }

}
